// Global.cpp: implementation of the CGlobal class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Global.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
CGlobal::CGlobal()
{
	for(int i=0; i<IDX_BMP_MAX; i++)
	{
		m_infoBitmap[i].hBitmap = NULL;
		::ZeroMemory(&m_infoBitmap[i].bitmap, sizeof(BITMAP));
	}

	for(int i=0; i<IDX_FONT_MAX; i++)
		m_pFont[i] = NULL;

	m_hIcon[0] = NULL;
	m_hIcon[1] = NULL;
	m_hIcon[2] = NULL;
	
	for(int i=0; i<IDX_RGB_MAX; i++)
		m_cf[i] = RGB(0,0,0);

	SetRGB(IDX_RGB_MASK, RGB(255,0,255));
	SetRGB(IDX_RGB_BACKGROUND, RGB(255,255,255));

	SetRGB(IDX_RGB_FONT, RGB(51,51,51));
	SetRGB(IDX_RGB_FONT_CAPTION, RGB(0,0,0));
	SetRGB(IDX_RGB_FONT_CAPTION_INACTIVE, RGB(178,178,178));

	//  get the function pointer for SetLayeredWindowAttributes in User32.dll
    HMODULE hUser32 = GetModuleHandle(_T("USER32.DLL"));
    SetLayeredWindowAttributes = (lpfn)GetProcAddress(hUser32, "SetLayeredWindowAttributes");
    //if(SetLayeredWindowAttributes == NULL)
	//	AfxMessageBox ("Layering is not supported in this version of Windows", MB_ICONEXCLAMATION);
}

CGlobal::~CGlobal()
{
	DeleteBitmaps();
	DeleteFonts();
	DeleteIcons();
}

void CGlobal::DeleteBitmaps()
{
	for(UINT i=0; i<IDX_BMP_MAX; i++)
	{
		if( m_infoBitmap[i].hBitmap )
		{
			::DeleteObject(m_infoBitmap[i].hBitmap);
			m_infoBitmap[i].hBitmap = NULL;
		}
		::ZeroMemory(&m_infoBitmap[i].bitmap, sizeof(BITMAP));
	}
}

void CGlobal::DeleteIcons()
{
	for(UINT k=0; k<3; k++)
	{
		if( m_hIcon[k] )
		{
			::DestroyIcon(m_hIcon[k]);
			m_hIcon[k] = NULL;
		}
	}
	
}

CString CGlobal::GetModulePath()
{
	CString sPath;
	GetModuleFileName( AfxGetApp()->m_hInstance, sPath.GetBuffer(MAX_PATH), MAX_PATH );
	sPath.ReleaseBuffer();

	int nPos = sPath.ReverseFind(_T('\\'));
	if( nPos != -1 )
		sPath = sPath.Left( nPos );

	return sPath;
}


SIZE CGlobal::GetBitmapSize(UINT nBitmapID)
{
	SIZE size;
	if( nBitmapID >= IDX_BMP_MAX )
	{
		size.cx=0;
		size.cy=0;
		return size;
	}

	size.cx = m_infoBitmap[nBitmapID].bitmap.bmWidth;
	size.cy = m_infoBitmap[nBitmapID].bitmap.bmHeight;
	return size;
}

HBITMAP CGlobal::GetBitmap(UINT nBitmapID)
{
	if( nBitmapID >= IDX_BMP_MAX )
		return NULL;

	return m_infoBitmap[nBitmapID].hBitmap;
}

HICON CGlobal::GetIcon( UINT nType)
{
	if( nType > ICON32 )
		return NULL;

	return m_hIcon[nType];
}

void CGlobal::DeleteFonts()
{
	for(UINT i=0; i<IDX_FONT_MAX; i++)
	{
		if( m_pFont[i] ) 
		{
			m_pFont[i]->DeleteObject();
			delete m_pFont[i];
			m_pFont[i] = NULL;
		}
	}
}

void CGlobal::SetRGB(UINT nRGBID, COLORREF cf)
{
	if( nRGBID >= IDX_RGB_MAX )
		return;

	m_cf[nRGBID] = cf;
}

COLORREF CGlobal::GetRGB(UINT nRGBID)
{
	if( nRGBID >= IDX_RGB_MAX )
		return RGB(0,0,0);

	return m_cf[nRGBID];
}

CFont * CGlobal::GetFont(UINT nFontID) 
{ 
	if( nFontID >= IDX_FONT_MAX )
		return NULL;

	return m_pFont[nFontID]; 
}

CFont * CGlobal::SetFont(UINT nFontID, CString sFaceName, UINT nFontSize, BOOL bBold/*=FALSE*/)
{
	if( nFontID >= IDX_FONT_MAX )
		return NULL;

	CWnd *pMain = AfxGetMainWnd();
	if( !pMain )
		return NULL;	

	// Font Name
	LOGFONT logFont;
	ZeroMemory(&logFont,sizeof(logFont));
	_tcscpy( logFont.lfFaceName, sFaceName );

	// Font Style
	logFont.lfItalic = FALSE;
	if( bBold )
		logFont.lfWeight = FW_BOLD;
	else
		logFont.lfWeight = FW_NORMAL;

	// Font Size
	HDC hdc = ::GetDC(pMain->m_hWnd);
	if( !hdc ) 
		logFont.lfHeight = nFontSize;
	else 
		logFont.lfHeight = -MulDiv(nFontSize, GetDeviceCaps(hdc, LOGPIXELSY), 72);
	::ReleaseDC(pMain->m_hWnd, hdc);

	// New Font
	if( m_pFont[nFontID] ) 
	{
		m_pFont[nFontID]->DeleteObject();
		delete m_pFont[nFontID];
		m_pFont[nFontID] = NULL;
	}

	m_pFont[nFontID] = (CFont *)new CFont;	
	if( !m_pFont[nFontID]->CreateFontIndirect( &logFont ) )
	{
		delete m_pFont[nFontID];
		m_pFont[nFontID] = NULL;
	}

	return m_pFont[nFontID];
}

CSize CGlobal::FormatText(CString& sText, CDC* pDC, int nWidth)
{
	CRect rect(0, 0, nWidth, 20);
	UINT uFlags = DT_CALCRECT | DT_SINGLELINE | DT_MODIFYSTRING | DT_END_ELLIPSIS;

	::DrawText(pDC->GetSafeHdc(), sText.GetBuffer(sText.GetLength() + 4), -1, rect, uFlags);
	sText.ReleaseBuffer();

	return pDC->GetTextExtent(sText);
}

COLORREF CGlobal::Darker(COLORREF crBase, float fFactor)
{
	ASSERT (fFactor < 1.0f && fFactor > 0.0f);

	fFactor = min(fFactor, 1.0f);
	fFactor = max(fFactor, 0.0f);

	BYTE bRed, bBlue, bGreen;
	BYTE bRedShadow, bBlueShadow, bGreenShadow;

	bRed = GetRValue(crBase);
	bBlue = GetBValue(crBase);
	bGreen = GetGValue(crBase);

	bRedShadow = (BYTE)(bRed * fFactor);
	bBlueShadow = (BYTE)(bBlue * fFactor);
	bGreenShadow = (BYTE)(bGreen * fFactor);

	return RGB(bRedShadow, bGreenShadow, bBlueShadow);
}

COLORREF CGlobal::Lighter(COLORREF crBase, float fFactor) 
{
	ASSERT (fFactor > 1.0f);

	fFactor = max(fFactor, 1.0f);

	BYTE bRed, bBlue, bGreen;
	BYTE bRedHilite, bBlueHilite, bGreenHilite;

	bRed = GetRValue(crBase);
	bBlue = GetBValue(crBase);
	bGreen = GetGValue(crBase);

	bRedHilite = (BYTE)min((int)(bRed * fFactor), 255);
	bBlueHilite = (BYTE)min((int)(bBlue * fFactor), 255);
	bGreenHilite = (BYTE)min((int)(bGreen * fFactor), 255);

	return RGB(bRedHilite, bGreenHilite, bBlueHilite);
}


BOOL CGlobal::SetLeftTopBitmap(CString strPath)
{
	m_infoBitmap[IDX_BMP_CAPTION_LEFT].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_CAPTION_LEFT].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_CAPTION_LEFT].bitmap);
	
	return 0;
}

BOOL CGlobal::SetCenterTopBitmap(CString strPath)
{
	m_infoBitmap[IDX_BMP_CAPTION_CENTER].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_CAPTION_CENTER].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_CAPTION_CENTER].bitmap);


	return 0;
}

BOOL CGlobal::SetRightTopBitmap(CString strPath)
{
	m_infoBitmap[IDX_BMP_CAPTION_RIGHT].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_CAPTION_RIGHT].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_CAPTION_RIGHT].bitmap);


	return 0;
}

BOOL CGlobal::SetLeftBorderBitmap(CString strPath)
{
	m_infoBitmap[IDX_BMP_FRAME_LEFT].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_FRAME_LEFT].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_FRAME_LEFT].bitmap);

	return 0;
}

BOOL CGlobal::SetRightBorderBitmap(CString strPath)
{
	m_infoBitmap[IDX_BMP_FRAME_RIGHT].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_FRAME_RIGHT].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_FRAME_RIGHT].bitmap);

	return 0;
}

BOOL CGlobal::SetLeftBottomBitmap(CString strPath)
{
	m_infoBitmap[IDX_BMP_FRAME_LEFT_BOTTOM].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_FRAME_LEFT_BOTTOM].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_FRAME_LEFT_BOTTOM].bitmap);

	return 0;
}

BOOL CGlobal::SetCenterBottomBitmap(CString strPath)
{
	m_infoBitmap[IDX_BMP_FRAME_BOTTOM].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_FRAME_BOTTOM].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_FRAME_BOTTOM].bitmap);

	return 0;
}

BOOL CGlobal::SetRightBottomBitmap(CString strPath)
{
	m_infoBitmap[IDX_BMP_FRAME_RIGHT_BOTTOM].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_FRAME_RIGHT_BOTTOM].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_FRAME_RIGHT_BOTTOM].bitmap);
	return 0;
}



// maxư ̹ 
BOOL CGlobal::SetMaxBitmaps(CString strNormalPath, CString strOverPath, CString strClickPath)
{
	//Max in Maximize
	m_infoBitmap[IDX_BMP_BOX_MAXMIZE].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strNormalPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_MAXMIZE].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_MAXMIZE].bitmap);



	m_infoBitmap[IDX_BMP_BOX_MAXMIZE_ON].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strClickPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_MAXMIZE_ON].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_MAXMIZE_ON].bitmap);

	m_infoBitmap[IDX_BMP_BOX_MAXMIZE_OVER].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strOverPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_MAXMIZE_OVER].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_MAXMIZE_OVER].bitmap);

	
	return TRUE;
}

BOOL CGlobal::SetRestoreBitmaps(CString strNormalPath, CString strOverPath, CString strClickPath)
{
	m_infoBitmap[IDX_BMP_BOX_RESTORE].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strNormalPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_RESTORE].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_RESTORE].bitmap);



	m_infoBitmap[IDX_BMP_BOX_RESTORE_ON].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strClickPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_RESTORE_ON].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_RESTORE_ON].bitmap);

	m_infoBitmap[IDX_BMP_BOX_RESTORE_OVER].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strOverPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_RESTORE_OVER].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_RESTORE_OVER].bitmap);


	return TRUE;
}

BOOL CGlobal::SetMinBitmaps(CString strNormalPath, CString strOverPath, CString strClickPath)
{
	m_infoBitmap[IDX_BMP_BOX_MINIMIZE].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strNormalPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_MINIMIZE].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_MINIMIZE].bitmap);



	m_infoBitmap[IDX_BMP_BOX_MINIMIZE_ON].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strClickPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_MINIMIZE_ON].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_MINIMIZE_ON].bitmap);

	m_infoBitmap[IDX_BMP_BOX_MINIMIZE_OVER].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strOverPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_MINIMIZE_OVER].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_MINIMIZE_OVER].bitmap);

	return TRUE;

}

BOOL CGlobal::SetExitBitmaps(CString strNormalPath, CString strOverPath, CString strClickPath)
{
	//exit in Maximize
	m_infoBitmap[IDX_BMP_BOX_EXIT].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strNormalPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_EXIT].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_EXIT].bitmap);



	m_infoBitmap[IDX_BMP_BOX_EXIT_ON].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strClickPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_EXIT_ON].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_EXIT_ON].bitmap);

	m_infoBitmap[IDX_BMP_BOX_EXIT_OVER].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strOverPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_EXIT_OVER].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_EXIT_OVER].bitmap);


	return TRUE;


}

BOOL CGlobal::SetCustomBitmaps(CString strNormalPath, CString strOverPath, CString strClickPath)
{
	//exit in Maximize
	m_infoBitmap[IDX_BMP_BOX_CUSTOM].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strNormalPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_CUSTOM].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_CUSTOM].bitmap);



	m_infoBitmap[IDX_BMP_BOX_CUSTOM_ON].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strClickPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_CUSTOM_ON].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_CUSTOM_ON].bitmap);

	m_infoBitmap[IDX_BMP_BOX_CUSTOM_OVER].hBitmap = (HBITMAP)::LoadImage(::AfxGetInstanceHandle(), 
		(LPCTSTR)strOverPath, 
		IMAGE_BITMAP, 
		0, 
		0, 
		LR_LOADFROMFILE|LR_CREATEDIBSECTION|LR_DEFAULTSIZE);

	::GetObject(m_infoBitmap[IDX_BMP_BOX_CUSTOM_OVER].hBitmap, sizeof(BITMAP), &m_infoBitmap[IDX_BMP_BOX_CUSTOM_OVER].bitmap);


	return TRUE;


}

void CGlobal::SetDlgIcon(CString strPath)
{
	DeleteIcons();
	m_hIcon[0] = (HICON)::LoadImage(::AfxGetInstanceHandle(), strPath, IMAGE_ICON, 16, 16, LR_LOADFROMFILE);
	m_hIcon[1] = (HICON)::LoadImage(::AfxGetInstanceHandle(), strPath, IMAGE_ICON, 24, 24, LR_LOADFROMFILE);
	m_hIcon[2] = (HICON)::LoadImage(::AfxGetInstanceHandle(), strPath, IMAGE_ICON, 32, 32, LR_LOADFROMFILE);

}

void CGlobal::SetDlgIcon(UINT nResourceID)
{
	DeleteIcons();
	m_hIcon[0] = AfxGetApp()->LoadIcon(nResourceID);
	m_hIcon[1] = AfxGetApp()->LoadIcon(nResourceID);
	m_hIcon[2] = AfxGetApp()->LoadIcon(nResourceID);

}
